#include "CalcPlugin.h"
#include <dlfcn.h>
#include <stdexcept>
#include <vector>


using mnd::CalcPlugin;

CalcPlugin::CalcPlugin(const std::string& path)
{
    handle = dlopen(path.c_str(), RTLD_LAZY);
}


CalcPlugin::~CalcPlugin(void)
{
    if (handle != nullptr) {
         dlclose(handle);
    }
}


const std::vector<mnd::MandelGenerator*>& CalcPlugin::getGenerators(void)
{
    static const std::vector<mnd::MandelGenerator*> empty = {};
    if (!isValid()) {
        return empty;
    }

    using GeneratorGetter =
        const typename std::vector<mnd::MandelGenerator*>& (*)(void);
    GeneratorGetter gg =
            GeneratorGetter(dlsym(handle, "mandel_get_generators"));
    if (gg != nullptr) {
        return gg();
    }
    else {
        return empty;
    }
}
